from instance.models import Instance
from instance.serializers import InstanceSerializer, FlavorSerializer
from django.http import Http404
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from interface_openstack.implementation.vm.instance import (
                                                        OSVirtualMachineManager
                                                        )
from template.models import InstanceTemplate
from instance.models import Instance, Flavor, Lease
from django.contrib.auth.models import User


class InstanceList(APIView):
    def get(self, request, format=None):
        instances = Instance.objects.all()
        return Response(InstanceSerializer(instances, many=True).data)

    def post(self, request, format=None):
        data = request.data
        template = InstanceTemplate(
                name="test1", description="adsasdasd",
                remote_ID="da51253f-867c-472d-8ce0-81e7b7126d60")
        flavor = Flavor(name="asd", description="asd", remote_id="1")
        flavor.save()
        lease = Lease(name="test_lease", description="test",
                      suspend_interval_in_sec=100,
                      delete_interval_in_sec=100)
        lease.save()

        newInstance = Instance.create_instance_from_template(
            params={"name": data["name"],
                    "description": data["description"],
                    "access_method": "ssh",
                    "system": data["system"],
                    "deleted": False,
                    "password": "asd",
                    },
            lease=lease,
            networks=[{"uuid": "c03d0d4b-413e-4cc6-9ebe-c0b5ca0dac3a"}],
            template=template,
            flavor=flavor,
            owner=User.objects.get(pk=1),
            disks="asdasd"
        )
        return Response(newInstance.pk)


class InstanceDetail(APIView):
    """
    Retrieve, update or delete a snippet instance.
    """

    def get_object(self, pk):
        try:
            return Instance.objects.get(pk=pk)
        except Instance.DoesNotExist:
            raise Http404

    def get(self, request, pk, format=None):
        instance = self.get_object(pk)
        instanceDict = InstanceSerializer(instance).data

        interface = OSVirtualMachineManager(settings.CONNECTION)
        remoteInstance = interface.get_vm(instance.remote_id)
        remoteInstanceDict = remoteInstance.__dict__

        merged_dict = {**instanceDict, **remoteInstanceDict}

        return Response(merged_dict)

    def put(self, request, pk, format=None):
        instance = self.get_object(pk)
        serializer = InstanceSerializer(instance, data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, pk, format=None):
        instance = self.get_object(pk)
        instance.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


class FlavorListView(APIView):
    """
    Create, update or delete a flavor.
    """

    def get(self, request, format=None):
        flavors = Flavor.objects.all()
        return Response(FlavorSerializer(flavors, many=True).data)

    def post(self, request, format=None):
        data = request.data
        new_flavor = Flavor.create(name=data["name"],
                                   description=data["description"],
                                   ram=data["ram"],
                                   vcpu=data["vcpu"],
                                   initial_disk=data["initial_disk"],
                                   priority=data["priority"])

        return Response(new_flavor.pk)
